;; Generic Collectivities model
;;
;; Nigel Gilbert 2006 - 2018
;;
;; November 2008 for NetLogo 4.0.3
;; March 2012 updated for NetLogo 5.0
;; April 2018 updated for NetLogo 6.0
;;
;; See http://www.lintar.disco.unimib.it/ABModSim/2006/gilbert.pdf
;; and Gilbert, Nigel (2008) Agent-Based Models.
;;  Quantitative Applications in the Social Sciences 153. SAGE Publications


breed [ agnts agnt ]

agnts-own [ around visible ]

to setup
  clear-all
  ; color the background white
  ask patches [set pcolor white ]
  ; create 500 agents and make them a little larger and green
  create-agnts 500 [
    set color green
    set size 2
    ; distribute agents randomly
    setxy random-pxcor random-pycor
    ; ensure that each is on its own patch
    be-alone
  ]
  ; start the clock at zero
  reset-ticks
end

to go
  ask agnts [ count-those-around ]
  ask agnts [ move ]
end

; store the number of agents surrounding me within local-radius units
; and the agents that I can see within visible-radius

to count-those-around
  set around count other agnts in-radius local-radius
  set visible other agnts in-radius visible-radius
end

to move
  if any? visible [
    ifelse around > threshold
      [ ; if more than threshold agents surround me, I'm crowded:
        ; face away from the most popular patch and become red
        face-away
        set color red ]
      [ ; else I'm lonely: face towards the most popular spot and become green
        face-towards
        set color green ]
    ]
  ; and move in my (new) direction
  forward random speed
  ; ensuring that I stop on an empty patch
  be-alone
end

; face towards the most popular local spot
to face-towards
  face max-one-of visible [around]
end

; face away from the most popular local spot
; by finding the direction of the most popular patch and
; then turning through 180 degrees to face away from it
to face-away
  set heading towards max-one-of visible [around] - 180
end

; move until I am on a patch with no other agents
to be-alone
  while [any? other agnts-here] [ forward 1 ]
end
@#$#@#$#@
GRAPHICS-WINDOW
215
10
589
385
-1
-1
6.0
1
10
1
1
1
0
1
1
1
-30
30
-30
30
0
0
1
ticks
30.0

BUTTON
35
20
101
53
NIL
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
130
20
193
53
NIL
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
30
180
204
213
threshold
threshold
0
100
23.0
1
1
NIL
HORIZONTAL

SLIDER
30
100
202
133
local-radius
local-radius
0
10
3.0
1
1
NIL
HORIZONTAL

SLIDER
30
220
202
253
speed
speed
0
20
5.0
1
1
NIL
HORIZONTAL

SLIDER
30
140
203
173
visible-radius
visible-radius
0
50
10.0
1
1
NIL
HORIZONTAL

TEXTBOX
32
275
182
312
Green: lonely agent\nRed: crowded agent
12
0.0
1

@#$#@#$#@
## WHAT IS IT?

The collectivities model consists of a surface over which agents are able to move. The surface is a toroid with each point representing one particular body of knowledge or set of beliefs. The agents thus move, not in a representation of physical space, but rather in 'knowledge space.' Although it may be oversimplifying to represent a knowledge space in two dimensions (more exactly, on the surface of a toroid), it makes for easier visualization.

An agent’s movement in the knowledge space represents its change in knowledge. Thus, if an agent imitates another agent, it would move toward that agent in the knowledge space, whereas if it innovates and discovers knowledge that other agents do not have, it would move away from other agents into previously empty areas.

Agents are initially distributed at random on the surface. Agents have no memory of their own or other agents’ previous positions. Each agent does the following:
  1. It counts how many other agents there are in its immediate neighborhood.
  2. If the number of agents is above a threshold, it turns to the direction opposite to the average direction of travel of other nearby agents and then moves a random distance.
  3. If the number of agents is equal to or below the threshold, it looks around the locality to find a relatively full area and then moves a random distance from its present location in the direction of the center of that area.

Each agent acts asynchronously, repeating this sequence of actions indefinitely. 


## HOW IT WORKS


```
breed [ agnts agnt ]
```
This line names the agent class that will be used in the model, in both its plural and singular forms (the name is not a mis-spelling: ‘agent’ is a reserved word in NetLogo and cannot be used as the name of a breed).
The agent class has two attributes, `around` and `visible`. Every agent has its own values for these variables, which store the number of other agents surrounding the agent and the set of agents that it can see, respectively.
```
to setup
clear-all
ask patches [set pcolor white ]
```
The setup procedure is executed when the user presses the Setup button on the interface. The procedure first deletes any agents left over from a previous run and then colors all the patches on the grid white. The code `ask patches [...]` tells all the patches (recall that a patch is a cell on the grid) to execute the code inside the square brackets. The term `pcolor` is a NetLogo variable that sets the color of a patch.
```
create-custom-agents 500 [
set color green
set size 2
```
Then 500 agents are created, each facing in a random direction. The commands inside the square brackets are executed by each of the new agents independently. The image of each agent on the view is colored green, and its size is set to twice the minimum.
```
setxy random-pxcor random-pycor
```
Then each agent is moved to a random spot on the grid (`random-pxcor` yields a number corresponding to a random patch coordinate between the left and right edges of the grid, and `random-pycor` does the same for a patch coordinate between the top and bottom edges).
We do not want more than one agent on a patch, and the procedure `be-alone` will test whether the agent is alone and if not, make it move forward until it lands on an empty patch.  This procedure is defined at the bottom of the code and we will see how it works in a moment.
```
]
reset-ticks
end
```
`reset-ticks` starts the tick counter.  There will be one tick of the clock for each step of the model run.  This completes the commands that are executed by each agent immediately after it is created, and also brings the `setup` procedure to an end.
```
to go
ask agents [ count-those-around ]
ask [ move ]
end
```
The `go` procedure gets executed repeatedly when the Go button on the interface is pressed. Each time through the procedure, the simulation moves on one time step (one tick). At each step, all the agents are asked to count the number of agents that are in their local area on the grid, and then all the agents are asked to move. It is important that all agents complete counting the agents around them before any agents move; otherwise, the counts may be confused by agents moving before they have been counted. NetLogo guarantees that all agents complete the commands in a construction such as `ask agnts[:::]` before the next command (in this case, the second `ask agnts[:::]`) is started. This would not have been the case if the program had been written `ask agnts [count-those-around move]` because then each agent would have started to move as soon as it had finished counting, and for some agents, this may have occurred before other agents had added them to their counts. This is an example of the kind of timing problem that one needs to guard against when programming agents that appear to act simultaneously.
The `count-those-around` and `move` procedures are both methods of the `agnt` class, that is, procedures that are defined in the program code to show what it means for agents to count and move. The next lines specify what `count-those-around` consists of.
```
to count-those-around
set around count agents with [self !=myself] in-radius local-radius
set visible agents with [self !=myself] in-radius visible-radius
end
```
In the first line of this procedure, the variable `around`, owned by each agent, is set to the number of agents that are located within a radius of `local-radius`. The user sets the value of `local-radius` before the simulation starts by moving a slider on the interface. It is easiest to see what this line of code does by working backwards from the end. The code `in-radius local-radius` yields those agents that are located at or within a distance of `local-radius` away from the agent executing this procedure. This set of agents includes the agent itself (it is zero units away from itself, and therefore closer than `local-radius`). The code `with [self != myself]` excludes that agent, and `count` returns the number of agents remaining. The agent’s attribute `around` is set to this number. One can think of the agent looking around, counting the number of agents it sees within the `local-radius` and remembering this in the `around` variable. 
The next line of code is very similar. It finds which other agents can be seen by the agent, that is, which agents are within the `visible-radius`, a parameter whose value is set by another slider on the interface. The second line is slightly different from the first because the variable, `visible`, stores not a count of the visible agents but the names of the agents themselves. This is necessary so that they can later be identified (in procedures `face-towards` and `face-away`).
The `move` procedure both decides the direction in which the agent is to move and performs the action.
```
to move
if any? visible [
```
The procedure begins by checking whether there are any agents visible to the agent executing this method. If there are, the agent needs to decide whether it is "lonely" or "crowded".
```
ifelse around>threshold
[
face-away
set color red ]
```
The condition consists of checking whether the number of agents around this agent is greater than the threshold parameter set with a slider on the user interface. If it is, it is "crowded" and the agent must turn away from the center of the crowd. The procedure face-away does this. In addition, to show on the view that it is “crowded”, its color is changed to red.

```
[
face-towards
set color green ]
]
```

The alternative is that the agent is "lonely," and it must turn toward the crowd and become green.  
Now that the agent is facing the right way, it should move forward, at a rate determined by the value of the `speed` parameter (`random speed` returns a random number less than the value of `speed`).
```
forward random speed
be-alone
end
```
As before, if the agent lands on a patch that is already occupied by another agent, it continues to move forward in the same direction until it finds an empty patch.
That concludes the `move` procedure. We still have to define `face-towards` and `face-away`.
```
to face-towards
face max-one-of visible [around]
end
```
The `face-towards` procedure retrieves the set of agents that this agent can see (the agents that have been remembered in the `visible` variable) and finds how many agents surround each of those agents. One of the agents with the most agents around it (`max-one-of visible [around]`) is considered to be at the center of the crowd toward which this agent wants to move. The command `face` sets the heading of the agent to the direction of that central agent.
```
to face-away
set heading towards max-one-of visible [around] - 180
end
```
The `face-away` procedure is almost the same, but the direction is 180 degrees opposite to the direction of the most central agent.
Finally, the `be-alone` procedure is defined.  
```
while [any? Other agnts-here] [ forward 1 ]
```
It is possible that two or more agents end up on the same patch, one standing on the head of another, so this procedure gets the agent to check whether there are any other agents on the patch. If there are, the agent advances by one unit (`forward 1`) in the direction of its current heading and checks again, continuing to advance until it has found an unoccupied patch. Because the grid is a toroid, if the agent moves off an edge, it will reappear on the opposite side.

## HOW TO USE IT

There are four parameters:
  1. The radius of the circular area surrounding an agent within which the number of agents is counted to determine whether the agent is "crowded" or "lonely" (`local-radius`)
  2. The threshold number of agents below which the agent is ``lonely’’ and above which the agent is ``crowded’’ (`threshold`)
  3. The radius of the circular area surrounding an agent in which the agent, if lonely, counts the number of agents to find where there is a maximum or, if crowded, finds the average direction of agent movement in order to determine the direction in which it is to move (`visible-radius`)
  4. The distance that an agent moves; the distance is chosen randomly from a uniform distribution with this parameter as the maximum (`speed`)

## THINGS TO NOTICE

Running the model shows that the initial uniform random distribution of agents separates into "clumps," in which some agents are central and others are distributed around them. The central agents are crowded, and so move. In doing so, they shift the centroid of the clump slightly and may make other agents either crowded or lonely, and they too will move. Thus, the clump of agents, although remaining together for long durations (as measured in time steps), drifts across the view. Lonely agents move toward the clump, sometimes joining it and sometimes continuing to trail behind it. The clumps never merge.

## THINGS TO TRY

Experiment to see whether the clumping behaviour is retained, no matter what the parameter settings are.


## CREDITS AND REFERENCES

This model is used as an example and further explained in Gilbert, Nigel (2008) _Agent-Based Models_. Quantitative Applications in the Social Sciences 153. SAGE Publications, and in the forthcoming second edition (SAGE, 2009).


 
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

link
true
0
Line -7500403 true 150 0 150 300

link direction
true
0
Line -7500403 true 150 150 30 225
Line -7500403 true 150 150 270 225

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270
@#$#@#$#@
NetLogo 6.0.1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180
@#$#@#$#@
0
@#$#@#$#@
